/*  scatter -- splits a file into multiple pieces by line
		 
    Copyright (C) 1985,1987,1991,1994,1996,2003  Norm London, London Computing

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	norm_london@londoncomputing.com

	Windows:     Compile with Visual Studio 5 or later or gcc 2.9.5 or later
	should work with just about any c compiler.
	cl scatter.c -DWIN32
	windows or unix using gcc:
	gcc scatter.c -o scatter
*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef WIN32
#define READ "rb"
#define WRITE "wb"
#define SEP "\\"
#else
#define WRITE "w"
#define READ "r"
#include <limits.h>
#define _MAX_PATH PATH_MAX
#define _MAX_DIR PATH_MAX
#define _MAX_DRIVE 3
#define _MAX_EXT PATH_MAX

#define SEP "/"


void _splitpath(char *path,char *drive,char *dir,char *fname,char *ext) {
	// for unix, ignore drive, set to null, use /
	char temp[PATH_MAX];
	int l;
	char *c,*t;


	*drive = 0;
	*dir = 0;
	*fname = 0;
	*ext = 0;

	if (strlen(path) > PATH_MAX) 
		//path is too long, just return all NULL
		return;
	else
		strcpy(temp,path);		
	
	l = strlen(temp) -1;
	t = temp;
	c = temp+l;
	while (l>0  && temp[l] != '.' && temp[l] != '/') {
//		fprintf(stderr,"1. c = %s\n",temp+l);
		--l;
	}
//	fprintf(stderr,"2. c = %s\n",temp+l);
	// found either start of string, . or /
	
	// if we found the .ext part, save it and keep going
	if (temp[l] == '.')	{
		strcpy(ext,temp+l);
		temp[l--] = 0;
//		fprintf(stderr,"3. ext = %s\n",ext);
	}

	// now look for the '/' character, we might already be there
	while (l > 0 && temp[l]!= '/') {
//		fprintf(stderr,"4. c = %s\n",temp+l);
		--l;
	}

	if (temp[l] == '/') {
		// save name part, w/o preceding / and continue
		strcpy(fname,temp+l+1);
		temp[l+1] = 0;
		strcpy(dir,temp);
//		fprintf(stderr,"5. fname = %s\ndir = %s\n",fname,dir);
		return;
	}

	// at start of string and no path specified because no /
	strcpy(fname,temp);
//	printf("6. fname = %s\next = %s\n",fname,ext);
	return;
}

void _makepath(char *path, char *drive, char *dir, char *fname, char *ext) {
	strcpy(path,drive); // this should be null
	strcat(path,dir);   // If not empty, should end with a /
	strcat(path,fname); // should never be empty
	strcat(path,ext);   // if not empty, should start with a .
	return;
}
#endif
	

#define MAXFILES 100
int main(int argc, char *argv[]) {
	// Usage is: scatter filename n
	// Creates n output files named: filename-xxx
	// where xxx is 3 digit file number 0-999.
	// reads filename and writes one line to
	// each output file, loops until done
	int number_of_output_files;
	char filename[_MAX_PATH], outpath[_MAX_PATH];
	char out_filename[_MAX_PATH];
	char drive[_MAX_DRIVE];
	char dir[_MAX_DIR];
	char name[_MAX_PATH], fname[_MAX_PATH];
	char ext[_MAX_PATH];
	char temp[_MAX_PATH];
	char buffer[65536];
	int maxfiles = MAXFILES;
	FILE *in,*out[MAXFILES];
	int i;

	for (i=0; i < MAXFILES; ++i)
		out[i] = NULL;

	if (argc < 3) {
		fprintf(stderr,"SCATTER V1.01 %s \n",__DATE__);
		fprintf(stderr,"Copyright 1998-2003, Norm London, London Computing.\nnorm_london@londoncomputing.com\n\n");
		fprintf(stderr,"This program is free software; you can redistribute it and/or modify\n");
		fprintf(stderr,"it under the terms of the GNU General Public License as published by\n");
		fprintf(stderr,"the Free Software Foundation; either version 2 of the License, or\n");
		fprintf(stderr,"(at your option) any later version.\n\n");
		fprintf(stderr,"This program is distributed in the hope that it will be useful,\n");
		fprintf(stderr,"but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
		fprintf(stderr,"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n");
		fprintf(stderr,"GNU General Public License for more details.\n\n");
		fprintf(stderr,"You should have received a copy of the GNU General Public License\n");
		fprintf(stderr,"along with this program; if not, write to the Free Software\n");
		fprintf(stderr,"Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\n\n");
		fprintf(stderr,"Usage is: SCATTER filename nnn [outputpath]\n");
		fprintf(stderr,"Where:\n");
		fprintf(stderr,"filename is the name of the input file.\n");
		fprintf(stderr,"nnn      is the number of output files.  nnn <= 1000\n");
		fprintf(stderr,"outpath  if present, specifies the name of the directory\n");
		fprintf(stderr,"         containing the ouput files.  If omitted, the\n");
		fprintf(stderr,"         output files are written to the directory\n");
		fprintf(stderr,"         containing the input file.\n\n");
		fprintf(stderr,"Scatter reads the input file and writes each line to\n");
		fprintf(stderr,"a different output file.  When the last output file has\n");
		fprintf(stderr,"been written, scatter starts over.  Every nth line is written\n");
		fprintf(stderr,"to the the same file.\n\n");
		fprintf(stderr,"Output Files are named: filename-nnn.ext where:\n");
		fprintf(stderr,"filename is the name of the input file and ext is\n");
		fprintf(stderr,"is the extention of the input file.\n\n");
		fprintf(stderr,"Example: scatter c:\\data\\input.dat 20 creates files:\n");
		fprintf(stderr,"c:\\data\\input-000.dat -- c:\\data\\input-019.dat\n");
		fprintf(stderr,"Returns 0 if OK, else 1.\n");
		return(1);
	}
	// get input file, number of output files
	strcpy(filename,argv[1]);
	in = fopen(filename,READ);
	if (in == NULL) {
		fprintf(stderr,"%s:  Unable to open input file.\n",argv[1]);
		return(1);
	}

	number_of_output_files = atoi(argv[2]);
	if (number_of_output_files <= 0 || number_of_output_files > MAXFILES) {
		fprintf(stderr,"%s:  Invalid file count.  Must be >0 and <=%d.\n",argv[2],maxfiles);
		return(1);
	}

	_splitpath(argv[1],drive,dir,fname,ext);

	if (argc >= 4) 
		strcpy(outpath,argv[3]);
	else {
		strcpy(outpath,drive);
		strcat(outpath,dir);
	}


	// read the file and scatter the records
	i = 0;
	while (fgets(buffer,sizeof(buffer),in)) {
		if (out[i] == NULL) {
			// make filename
			strcpy(out_filename,outpath);
			strcpy(name,fname);
			strcat(name,"-");
			sprintf(temp,"%3d",i);
			if (temp[0]==' ')
				temp[0]='0';
			if (temp[1]==' ')
				temp[1]='0';
			strcat(name,temp);
			if (*out_filename)
				strcat(out_filename,SEP);
			strcat(out_filename,name);
			strcat(out_filename,ext);
//			_makepath(out_filename,drive,dir,name,ext);
			// open file
			out[i] = fopen(out_filename,WRITE);
			if (out[i] == NULL) {
				fprintf(stderr,"Unable to open output file %d\n",i+1);
				return(1);
			}
		}
		fputs(buffer,out[i]);
		if (++i >= number_of_output_files)
			i = 0;
	}
	fclose(in);
	for (i=0; i < number_of_output_files; ++i)
		if (out[i])
			fclose(out[i]);
	return(0);
}


